/* USER CODE BEGIN Header */
/**
 ******************************************************************************
 * @file           : main.c
 * @brief          : Main program body
 ******************************************************************************
 * @attention
 *
 * <h2><center>&copy; Copyright (c) 2020 STMicroelectronics.
 * All rights reserved.</center></h2>
 *
 * This software component is licensed by ST under BSD 3-Clause license,
 * the "License"; You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at:
 *                        opensource.org/licenses/BSD-3-Clause
 *
 ******************************************************************************
 */
/* USER CODE END Header */

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include <string.h>
#include <stdio.h>
#include "hts221_reg.h"
#include "lps22hb_reg.h"
#include "lsm303agr_reg.h"
#include "lsm6dsl_reg.h"
#include "lcd.h"

/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */
typedef union {
	int16_t i16bit;
	uint8_t u8bit[2];
} axis1bit16_t;


typedef union{
  int32_t i32bit;
  uint8_t u8bit[4];
} axis1bit32_t;

typedef union{
  int16_t i16bit[3];
  uint8_t u8bit[6];
} axis3bit16_t;


typedef union{
  int32_t i32bit[3];
  uint8_t u8bit[12];
} axis3bit32_t;

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */
#define NUCLEO_F767ZI
#define TX_BUF_DIM          1000

/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/

I2C_HandleTypeDef hi2c1;

UART_HandleTypeDef huart3;

/* USER CODE BEGIN PV */
static axis1bit16_t data_raw_humidity;
static axis1bit16_t data_raw_temperature;
static axis1bit32_t data_raw_pressure;
static axis3bit16_t data_raw_acceleration;
static axis3bit16_t data_raw_magnetic;
static axis3bit16_t data_raw_angular_rate;
static float humidity_perc;
static float temperature_degC;
static float pressure_hPa;
static float acceleration_mg[3];
static float magnetic_mG[3];
static float angular_rate_mdps[3];
static uint8_t whoamI, rst;
static uint8_t tx_buffer[TX_BUF_DIM];

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
static void MX_GPIO_Init(void);
static void MX_I2C1_Init(void);
static void MX_USART3_UART_Init(void);
/* USER CODE BEGIN PFP */
static int32_t hts221_platform_write(void *handle, uint8_t reg, uint8_t *bufp,
		uint16_t len);
static int32_t hts221_platform_read(void *handle, uint8_t reg, uint8_t *bufp,
		uint16_t len);
static int32_t lps22hb_platform_write(void *handle, uint8_t reg, uint8_t *bufp,
		uint16_t len);
static int32_t lps22hb_platform_read(void *handle, uint8_t reg, uint8_t *bufp,
		uint16_t len);
static int32_t lsm303agr_platform_write(void *handle, uint8_t reg, uint8_t *bufp,
		uint16_t len);
static int32_t lsm303agr_platform_read(void *handle, uint8_t reg, uint8_t *bufp,
		uint16_t len);
static int32_t lsm6dsl_platform_write(void *handle, uint8_t reg, uint8_t *bufp,
		uint16_t len);
static int32_t lsm6dsl_platform_read(void *handle, uint8_t reg, uint8_t *bufp,
		uint16_t len);
static void tx_com(uint8_t *tx_buffer, uint16_t len);
static void platform_delay(uint32_t ms);
static void platform_init(void);

/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */
typedef struct {
	float x0;
	float y0;
	float x1;
	float y1;
} lin_t;

float linear_interpolation(lin_t *lin, int16_t x) {
	return ((lin->y1 - lin->y0) * x
			+ ((lin->x1 * lin->y0) - (lin->x0 * lin->y1))) / (lin->x1 - lin->x0);
}

/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */
  int sens = 0;
  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_I2C1_Init();
  MX_USART3_UART_Init();
  /* USER CODE BEGIN 2 */

  lcdInit(&hi2c1);

  /* Initialize platform specific hardware */
  platform_init();

  /* Initialize mems driver interface */
  stmdev_ctx_t hts221_dev_ctx;
  hts221_dev_ctx.write_reg = hts221_platform_write;
  hts221_dev_ctx.read_reg = hts221_platform_read;
  hts221_dev_ctx.handle = &hi2c1;

  stmdev_ctx_t lps22hb_dev_ctx;
  lps22hb_dev_ctx.write_reg = lps22hb_platform_write;
  lps22hb_dev_ctx.read_reg = lps22hb_platform_read;
  lps22hb_dev_ctx.handle = &hi2c1;

  stmdev_ctx_t lsm303agr_dev_ctx_xl;
  lsm303agr_dev_ctx_xl.write_reg = lsm303agr_platform_write;
  lsm303agr_dev_ctx_xl.read_reg = lsm303agr_platform_read;
  lsm303agr_dev_ctx_xl.handle = (void*)LSM303AGR_I2C_ADD_XL;

  stmdev_ctx_t lsm303agr_dev_ctx_mg;
  lsm303agr_dev_ctx_mg.write_reg = lsm303agr_platform_write;
  lsm303agr_dev_ctx_mg.read_reg = lsm303agr_platform_read;
  lsm303agr_dev_ctx_mg.handle = (void*)LSM303AGR_I2C_ADD_MG;

  stmdev_ctx_t lsm6dsl_dev_ctx;
  lsm6dsl_dev_ctx.write_reg = lsm6dsl_platform_write;
  lsm6dsl_dev_ctx.read_reg = lsm6dsl_platform_read;
  lsm6dsl_dev_ctx.handle = &hi2c1;

  /* Check device ID */
  whoamI = 0;
  hts221_device_id_get(&hts221_dev_ctx, &whoamI);
  if (whoamI != HTS221_ID)
	  while (1)
		  ; /*manage here device not found */
  whoamI = 0;
  lps22hb_device_id_get(&lps22hb_dev_ctx, &whoamI);
  if (whoamI != LPS22HB_ID){
	  while(1)
		  ; /* manage here device not found */
  }
  whoamI = 0;
  lsm303agr_xl_device_id_get(&lsm303agr_dev_ctx_xl, &whoamI);
  if ( whoamI != LSM303AGR_ID_XL )
	  while(1)
		  ; /*manage here device not found */
  whoamI = 0;
  lsm303agr_mag_device_id_get(&lsm303agr_dev_ctx_mg, &whoamI);
  if ( whoamI != LSM303AGR_ID_MG )
	  while(1)
		  ; /*manage here device not found */
  whoamI = 0;
  lsm6dsl_device_id_get(&lsm6dsl_dev_ctx, &whoamI);
  if ( whoamI != LSM6DSL_ID )
    while(1)
    	; /*manage here device not found */


  /* Read humidity calibration coefficient */
  lin_t lin_hum;
  hts221_hum_adc_point_0_get(&hts221_dev_ctx, &lin_hum.x0);
  hts221_hum_rh_point_0_get(&hts221_dev_ctx, &lin_hum.y0);
  hts221_hum_adc_point_1_get(&hts221_dev_ctx, &lin_hum.x1);
  hts221_hum_rh_point_1_get(&hts221_dev_ctx, &lin_hum.y1);

  /* Read temperature calibration coefficient */
  lin_t lin_temp;
  hts221_temp_adc_point_0_get(&hts221_dev_ctx, &lin_temp.x0);
  hts221_temp_deg_point_0_get(&hts221_dev_ctx, &lin_temp.y0);
  hts221_temp_adc_point_1_get(&hts221_dev_ctx, &lin_temp.x1);
  hts221_temp_deg_point_1_get(&hts221_dev_ctx, &lin_temp.y1);

  /* Enable Block Data Update */
  hts221_block_data_update_set(&hts221_dev_ctx, PROPERTY_ENABLE);
  /* Set Output Data Rate */
  hts221_data_rate_set(&hts221_dev_ctx, HTS221_ODR_1Hz);
  /* Device power on */
  hts221_power_on_set(&hts221_dev_ctx, PROPERTY_ENABLE);

  /* Restore default configuration */
  lps22hb_reset_set(&lps22hb_dev_ctx, PROPERTY_ENABLE);
  do {
	  lps22hb_reset_get(&lps22hb_dev_ctx, &rst);
  } while (rst);

  /* Enable Block Data Update */
  //lps22hb_block_data_update_set(&lps22hb_dev_ctx, PROPERTY_ENABLE);

  /* Can be enabled low pass filter on output */
  lps22hb_low_pass_filter_mode_set(&lps22hb_dev_ctx, LPS22HB_LPF_ODR_DIV_2);

  /* Can be set Data-ready signal on INT_DRDY pin */
  //lps22hb_drdy_on_int_set(&lps22hb_dev_ctx, PROPERTY_ENABLE);

  /* Set Output Data Rate */
  lps22hb_data_rate_set(&lps22hb_dev_ctx, LPS22HB_ODR_10_Hz);

  /* Restore default configuration for magnetometer */
  lsm303agr_mag_reset_set(&lsm303agr_dev_ctx_mg, PROPERTY_ENABLE);
  do {
	  lsm303agr_mag_reset_get(&lsm303agr_dev_ctx_mg, &rst);
  } while (rst);

  /* Enable Block Data Update */
  lsm303agr_xl_block_data_update_set(&lsm303agr_dev_ctx_xl, PROPERTY_ENABLE);
  lsm303agr_mag_block_data_update_set(&lsm303agr_dev_ctx_mg, PROPERTY_ENABLE);
  /* Set Output Data Rate */
  lsm303agr_xl_data_rate_set(&lsm303agr_dev_ctx_xl, LSM303AGR_XL_ODR_1Hz);
  lsm303agr_mag_data_rate_set(&lsm303agr_dev_ctx_mg, LSM303AGR_MG_ODR_10Hz);
  /* Set accelerometer full scale */
  lsm303agr_xl_full_scale_set(&lsm303agr_dev_ctx_xl, LSM303AGR_2g);
  /* Set / Reset magnetic sensor mode */
  lsm303agr_mag_set_rst_mode_set(&lsm303agr_dev_ctx_mg, LSM303AGR_SENS_OFF_CANC_EVERY_ODR);
  /* Enable temperature compensation on mag sensor */
  lsm303agr_mag_offset_temp_comp_set(&lsm303agr_dev_ctx_mg, PROPERTY_ENABLE);
  /* Enable temperature sensor */
  lsm303agr_temperature_meas_set(&lsm303agr_dev_ctx_xl, LSM303AGR_TEMP_ENABLE);
  /* Set device in continuos mode */
  lsm303agr_xl_operating_mode_set(&lsm303agr_dev_ctx_xl, LSM303AGR_HR_12bit);
  /* Set magnetometer in continuos mode */
  lsm303agr_mag_operating_mode_set(&lsm303agr_dev_ctx_mg, LSM303AGR_CONTINUOUS_MODE);

  /*
   *  Restore default configuration
   */
  lsm6dsl_reset_set(&lsm6dsl_dev_ctx, PROPERTY_ENABLE);
  do {
    lsm6dsl_reset_get(&lsm6dsl_dev_ctx, &rst);
  } while (rst);
  /*
   *  Enable Block Data Update
   */
  lsm6dsl_block_data_update_set(&lsm6dsl_dev_ctx, PROPERTY_ENABLE);
  /*
   * Set Output Data Rate
   */
  lsm6dsl_xl_data_rate_set(&lsm6dsl_dev_ctx, LSM6DSL_XL_ODR_12Hz5);
  lsm6dsl_gy_data_rate_set(&lsm6dsl_dev_ctx, LSM6DSL_GY_ODR_12Hz5);
  /*
   * Set full scale
   */
  lsm6dsl_xl_full_scale_set(&lsm6dsl_dev_ctx, LSM6DSL_2g);
  lsm6dsl_gy_full_scale_set(&lsm6dsl_dev_ctx, LSM6DSL_2000dps);

  /*
   * Configure filtering chain(No aux interface)
   */
  /* Accelerometer - analog filter */
  lsm6dsl_xl_filter_analog_set(&lsm6dsl_dev_ctx, LSM6DSL_XL_ANA_BW_400Hz);

  /* Accelerometer - LPF1 path ( LPF2 not used )*/
  //lsm6dsl_xl_lp1_bandwidth_set(&lsm6dsl_dev_ctx, LSM6DSL_XL_LP1_ODR_DIV_4);

  /* Accelerometer - LPF1 + LPF2 path */
  lsm6dsl_xl_lp2_bandwidth_set(&lsm6dsl_dev_ctx, LSM6DSL_XL_LOW_NOISE_LP_ODR_DIV_100);

  /* Accelerometer - High Pass / Slope path */
  //lsm6dsl_xl_reference_mode_set(&lsm6dsl_dev_ctx, PROPERTY_DISABLE);
  //lsm6dsl_xl_hp_bandwidth_set(&lsm6dsl_dev_ctx, LSM6DSL_XL_HP_ODR_DIV_100);

  /* Gyroscope - filtering chain */
  lsm6dsl_gy_band_pass_set(&lsm6dsl_dev_ctx, LSM6DSL_HP_260mHz_LP1_STRONG);

  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
	while (1) {
		/* hts221 */
		if (sens == 0) {
			/* Read output only if new value is available */
			hts221_reg_t reg;
			hts221_status_get(&hts221_dev_ctx, &reg.status_reg);

			if (reg.status_reg.h_da) {
				/* Read humidity data */
				memset(data_raw_humidity.u8bit, 0x00, sizeof(int16_t));
				hts221_humidity_raw_get(&hts221_dev_ctx, data_raw_humidity.u8bit);
				humidity_perc = linear_interpolation(&lin_hum,
						data_raw_humidity.i16bit);
				if (humidity_perc < 0)
					humidity_perc = 0;
				if (humidity_perc > 100)
					humidity_perc = 100;
				sprintf((char*) tx_buffer, "Hum[perc]:%6.2f",
						humidity_perc);
				HAL_Delay(100);
				tx_com(tx_buffer, strlen((char const*) tx_buffer));
				lcdSetCursor(0,0);
				lcdPrint(tx_buffer);
			}
			if (reg.status_reg.t_da) {
				/* Read temperature data */
				memset(data_raw_temperature.u8bit, 0x00, sizeof(int16_t));
				hts221_temperature_raw_get(&hts221_dev_ctx, data_raw_temperature.u8bit);
				temperature_degC = linear_interpolation(&lin_temp,
						data_raw_temperature.i16bit);
				sprintf((char*) tx_buffer, "Temp[dgC]:%6.2f",
						temperature_degC);
				HAL_Delay(100);
				tx_com(tx_buffer, strlen((char const*) tx_buffer));
				lcdSetCursor(0,1);
				lcdPrint(tx_buffer);
			}
		}

		/* lps22hb */
		if (sens == 1) {
			uint8_t reg;

			/* Read output only if new value is available */
			lps22hb_press_data_ready_get(&lps22hb_dev_ctx, &reg);
			if (reg)
			{
				memset(data_raw_pressure.u8bit, 0x00, sizeof(int32_t));
				lps22hb_pressure_raw_get(&lps22hb_dev_ctx, data_raw_pressure.u8bit);

				pressure_hPa = lps22hb_from_lsb_to_hpa(data_raw_pressure.i32bit);
				sprintf((char*)tx_buffer, "Prs[hPa]:%6.2f", pressure_hPa);

				tx_com(tx_buffer, strlen((char const*)tx_buffer));
				HAL_Delay(100);
				lcdSetCursor(0,0);
				lcdPrint(tx_buffer);

				memset(data_raw_temperature.u8bit, 0x00, sizeof(int16_t));
				lps22hb_temperature_raw_get(&lps22hb_dev_ctx, data_raw_temperature.u8bit);

				temperature_degC = lps22hb_from_lsb_to_degc(data_raw_temperature.i16bit);
				sprintf((char*)tx_buffer, "Temp[dgC]:%6.2f", temperature_degC);
				HAL_Delay(100);
				tx_com(tx_buffer, strlen((char const*)tx_buffer));
				lcdSetCursor(0,1);
				lcdPrint(tx_buffer);
			}
		}

		/* lsm303agr */
		if (sens == 2) {
		    /* Read output only if new value is available */
		    lsm303agr_reg_t reg;
		    lsm303agr_xl_status_get(&lsm303agr_dev_ctx_xl, &reg.status_reg_a);

		    if (reg.status_reg_a.zyxda)
		    {
		      /* Read accelerometer data */
		      memset(data_raw_acceleration.u8bit, 0x00, 3*sizeof(int16_t));
		      lsm303agr_acceleration_raw_get(&lsm303agr_dev_ctx_xl, data_raw_acceleration.u8bit);
		      acceleration_mg[0] = lsm303agr_from_fs_2g_hr_to_mg( data_raw_acceleration.i16bit[0] );
		      acceleration_mg[1] = lsm303agr_from_fs_2g_hr_to_mg( data_raw_acceleration.i16bit[1] );
		      acceleration_mg[2] = lsm303agr_from_fs_2g_hr_to_mg( data_raw_acceleration.i16bit[2] );

		      sprintf((char*)tx_buffer, "Acc[mg]:%4.2f",
		              acceleration_mg[0]);
		      tx_com( tx_buffer, strlen( (char const*)tx_buffer ) );
		      HAL_Delay(100);
		      lcdSetCursor(0,0);
		      lcdPrint(tx_buffer);
		      sprintf((char*)tx_buffer, "%4.2f %4.2f",
		              acceleration_mg[1], acceleration_mg[2]);
		      tx_com( tx_buffer, strlen( (char const*)tx_buffer ) );
		      HAL_Delay(100);
		      lcdSetCursor(0,1);
		      lcdPrint(tx_buffer);
		    }
		}

		if (sens == 3) {
			/* Read output only if new value is available */
			lsm303agr_reg_t reg;

		    lsm303agr_mag_status_get(&lsm303agr_dev_ctx_mg, &reg.status_reg_m);
		    if (reg.status_reg_m.zyxda)
		    {
		      /* Read magnetic field data */
		      memset(data_raw_magnetic.u8bit, 0x00, 3*sizeof(int16_t));
		      lsm303agr_magnetic_raw_get(&lsm303agr_dev_ctx_mg, data_raw_magnetic.u8bit);
		      magnetic_mG[0] = lsm303agr_from_lsb_to_mgauss( data_raw_magnetic.i16bit[0]);
		      magnetic_mG[1] = lsm303agr_from_lsb_to_mgauss( data_raw_magnetic.i16bit[1]);
		      magnetic_mG[2] = lsm303agr_from_lsb_to_mgauss( data_raw_magnetic.i16bit[2]);

		      sprintf((char*)tx_buffer, "Mag[mG]:%4.2f",
		              magnetic_mG[0]);
		      tx_com( tx_buffer, strlen( (char const*)tx_buffer ) );
		      HAL_Delay(100);
		      lcdSetCursor(0,0);
		      lcdPrint(tx_buffer);
		      sprintf((char*)tx_buffer, "%4.2f %4.2f",
		              magnetic_mG[1], magnetic_mG[2]);
		      tx_com( tx_buffer, strlen( (char const*)tx_buffer ) );
		      HAL_Delay(100);
		      lcdSetCursor(0,1);
		      lcdPrint(tx_buffer);
		    }
		}

		if (sens == 4) {
			/* Read output only if new value is available */
			lsm303agr_reg_t reg;

			lsm303agr_temp_data_ready_get(&lsm303agr_dev_ctx_xl, &reg.byte);
			if (reg.byte)
			{
				/* Read temperature data */
				memset(data_raw_temperature.u8bit, 0x00, sizeof(int16_t));
				lsm303agr_temperature_raw_get(&lsm303agr_dev_ctx_xl, data_raw_temperature.u8bit);
				temperature_degC = lsm303agr_from_lsb_hr_to_celsius( data_raw_temperature.i16bit );

				sprintf((char*)tx_buffer, "Temp[dgC]:%6.2f", temperature_degC );
				tx_com( tx_buffer, strlen( (char const*)tx_buffer ) );
				HAL_Delay(100);
				lcdSetCursor(0,0);
				lcdPrint(tx_buffer);
				sprintf((char*)tx_buffer, "                ");
				tx_com( tx_buffer, strlen( (char const*)tx_buffer ) );
				HAL_Delay(100);
				lcdSetCursor(0,1);
				lcdPrint(tx_buffer);
			}
		}

		/* lsm6dsl */
		if (sens == 5) {
		    /*
		     * Read output only if new value is available
		     */
		    lsm6dsl_reg_t reg;
		    lsm6dsl_status_reg_get(&lsm6dsl_dev_ctx, &reg.status_reg);

		    if (reg.status_reg.xlda)
		    {
		      /* Read magnetic field data */
		      memset(data_raw_acceleration.u8bit, 0x00, 3*sizeof(int16_t));
		      lsm6dsl_acceleration_raw_get(&lsm6dsl_dev_ctx, data_raw_acceleration.u8bit);
		      acceleration_mg[0] = lsm6dsl_from_fs2g_to_mg( data_raw_acceleration.i16bit[0]);
		      acceleration_mg[1] = lsm6dsl_from_fs2g_to_mg( data_raw_acceleration.i16bit[1]);
		      acceleration_mg[2] = lsm6dsl_from_fs2g_to_mg( data_raw_acceleration.i16bit[2]);

		      sprintf((char*)tx_buffer, "Acc[mg]:%4.2f",
		              acceleration_mg[0]);
		      tx_com( tx_buffer, strlen( (char const*)tx_buffer ) );
		      HAL_Delay(100);
		      lcdSetCursor(0,0);
		      lcdPrint(tx_buffer);
		      sprintf((char*)tx_buffer, "%4.2f %4.2f",
		              acceleration_mg[1], acceleration_mg[2]);
		      tx_com( tx_buffer, strlen( (char const*)tx_buffer ) );
		      HAL_Delay(100);
		      lcdSetCursor(0,1);
		      lcdPrint(tx_buffer);
		    }
		}

		if (sens == 6) {
		    /*
		     * Read output only if new value is available
		     */
		    lsm6dsl_reg_t reg;
		    lsm6dsl_status_reg_get(&lsm6dsl_dev_ctx, &reg.status_reg);
		    if (reg.status_reg.gda)
		    {
		      /* Read magnetic field data */
		      memset(data_raw_angular_rate.u8bit, 0x00, 3*sizeof(int16_t));
		      lsm6dsl_angular_rate_raw_get(&lsm6dsl_dev_ctx, data_raw_angular_rate.u8bit);
		      angular_rate_mdps[0] = lsm6dsl_from_fs2000dps_to_mdps(data_raw_angular_rate.i16bit[0]);
		      angular_rate_mdps[1] = lsm6dsl_from_fs2000dps_to_mdps(data_raw_angular_rate.i16bit[1]);
		      angular_rate_mdps[2] = lsm6dsl_from_fs2000dps_to_mdps(data_raw_angular_rate.i16bit[2]);

		      sprintf((char*)tx_buffer, "Angrate[mdps]:%4.2f",
		              angular_rate_mdps[0]);
		      tx_com( tx_buffer, strlen( (char const*)tx_buffer ) );
		      HAL_Delay(100);
		      lcdSetCursor(0,0);
		      lcdPrint(tx_buffer);
		      sprintf((char*)tx_buffer, "%4.2f %4.2f",
		    		  angular_rate_mdps[1], angular_rate_mdps[2]);
		      tx_com( tx_buffer, strlen( (char const*)tx_buffer ) );
		      HAL_Delay(100);
		      lcdSetCursor(0,1);
		      lcdPrint(tx_buffer);
		    }
		}

		if (sens == 7) {
		    /*
		     * Read output only if new value is available
		     */
		    lsm6dsl_reg_t reg;
		    lsm6dsl_status_reg_get(&lsm6dsl_dev_ctx, &reg.status_reg);
		    if (reg.status_reg.tda)
		    {
		      /* Read temperature data */
		      memset(data_raw_temperature.u8bit, 0x00, sizeof(int16_t));
		      lsm6dsl_temperature_raw_get(&lsm6dsl_dev_ctx, data_raw_temperature.u8bit);
		      temperature_degC = lsm6dsl_from_lsb_to_celsius( data_raw_temperature.i16bit );

		      sprintf((char*)tx_buffer, "Temp[dgC]:%6.2f", temperature_degC );
		      tx_com( tx_buffer, strlen( (char const*)tx_buffer ) );
		      HAL_Delay(100);
		      lcdSetCursor(0,0);
		      lcdPrint(tx_buffer);
		      sprintf((char*)tx_buffer, "                ");
		      tx_com( tx_buffer, strlen( (char const*)tx_buffer ) );
		      HAL_Delay(100);
		      lcdSetCursor(0,1);
		      lcdPrint(tx_buffer);
		    }

		}

    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */
		uint8_t but = lcdReadButtons();
		if (!(but & 0x01)) {
			sens = 0;
		} else if (!(but & 0x02)) {
			sens = 1;
		} else if (!(but & 0x04) && sens != 2 && sens != 3) {
			sens = 2;
		} else if (!(but & 0x04) && sens == 2) {
			sens = 3;
		} else if (!(but & 0x04) && sens == 3) {
			sens = 4;
		} else if (!(but & 0x08) && sens != 5 && sens != 6) {
			sens = 5;
		} else if (!(but & 0x08) && sens == 5) {
			sens = 6;
		} else if (!(but & 0x08) && sens == 6) {
			sens = 7;
		} //else, unchanged
	}
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};
  RCC_PeriphCLKInitTypeDef PeriphClkInitStruct = {0};

  /** Configure the main internal regulator output voltage 
  */
  __HAL_RCC_PWR_CLK_ENABLE();
  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE3);
  /** Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.HSICalibrationValue = RCC_HSICALIBRATION_DEFAULT;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_NONE;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }
  /** Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_HSI;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_0) != HAL_OK)
  {
    Error_Handler();
  }
  PeriphClkInitStruct.PeriphClockSelection = RCC_PERIPHCLK_USART3|RCC_PERIPHCLK_I2C1;
  PeriphClkInitStruct.Usart3ClockSelection = RCC_USART3CLKSOURCE_PCLK1;
  PeriphClkInitStruct.I2c1ClockSelection = RCC_I2C1CLKSOURCE_PCLK1;
  if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInitStruct) != HAL_OK)
  {
    Error_Handler();
  }
}

/**
  * @brief I2C1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_I2C1_Init(void)
{

  /* USER CODE BEGIN I2C1_Init 0 */

  /* USER CODE END I2C1_Init 0 */

  /* USER CODE BEGIN I2C1_Init 1 */

  /* USER CODE END I2C1_Init 1 */
  hi2c1.Instance = I2C1;
  hi2c1.Init.Timing = 0x00303D5B;
  hi2c1.Init.OwnAddress1 = 0;
  hi2c1.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
  hi2c1.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
  hi2c1.Init.OwnAddress2 = 0;
  hi2c1.Init.OwnAddress2Masks = I2C_OA2_NOMASK;
  hi2c1.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
  hi2c1.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;
  if (HAL_I2C_Init(&hi2c1) != HAL_OK)
  {
    Error_Handler();
  }
  /** Configure Analogue filter 
  */
  if (HAL_I2CEx_ConfigAnalogFilter(&hi2c1, I2C_ANALOGFILTER_ENABLE) != HAL_OK)
  {
    Error_Handler();
  }
  /** Configure Digital filter 
  */
  if (HAL_I2CEx_ConfigDigitalFilter(&hi2c1, 0) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN I2C1_Init 2 */

  /* USER CODE END I2C1_Init 2 */

}

/**
  * @brief USART3 Initialization Function
  * @param None
  * @retval None
  */
static void MX_USART3_UART_Init(void)
{

  /* USER CODE BEGIN USART3_Init 0 */

  /* USER CODE END USART3_Init 0 */

  /* USER CODE BEGIN USART3_Init 1 */

  /* USER CODE END USART3_Init 1 */
  huart3.Instance = USART3;
  huart3.Init.BaudRate = 115200;
  huart3.Init.WordLength = UART_WORDLENGTH_8B;
  huart3.Init.StopBits = UART_STOPBITS_1;
  huart3.Init.Parity = UART_PARITY_NONE;
  huart3.Init.Mode = UART_MODE_TX_RX;
  huart3.Init.HwFlowCtl = UART_HWCONTROL_NONE;
  huart3.Init.OverSampling = UART_OVERSAMPLING_16;
  huart3.Init.OneBitSampling = UART_ONE_BIT_SAMPLE_DISABLE;
  huart3.AdvancedInit.AdvFeatureInit = UART_ADVFEATURE_NO_INIT;
  if (HAL_UART_Init(&huart3) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN USART3_Init 2 */

  /* USER CODE END USART3_Init 2 */

}

/**
  * @brief GPIO Initialization Function
  * @param None
  * @retval None
  */
static void MX_GPIO_Init(void)
{

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOD_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();

}

/* USER CODE BEGIN 4 */


/*
 * @brief  Write generic device register (platform dependent)
 *
 * @param  handle    customizable argument. In this examples is used in
 *                   order to select the correct sensor bus handler.
 * @param  reg       register to write
 * @param  bufp      pointer to data to write in register reg
 * @param  len       number of consecutive register to write
 *
 */
static int32_t hts221_platform_write(void *handle, uint8_t reg, uint8_t *bufp,
		uint16_t len) {
	if (handle == &hi2c1) {
		/* Write multiple command */
		reg |= 0x80;
		HAL_I2C_Mem_Write(handle, HTS221_I2C_ADDRESS, reg,
		I2C_MEMADD_SIZE_8BIT, bufp, len, 1000);
	}
	return 0;
}

/*
 * @brief  Read generic device register (platform dependent)
 *
 * @param  handle    customizable argument. In this examples is used in
 *                   order to select the correct sensor bus handler.
 * @param  reg       register to read
 * @param  bufp      pointer to buffer that store the data read
 * @param  len       number of consecutive register to read
 *
 */
static int32_t hts221_platform_read(void *handle, uint8_t reg, uint8_t *bufp,
		uint16_t len) {
	if (handle == &hi2c1) {
		/* Read multiple command */
		reg |= 0x80;
		HAL_I2C_Mem_Read(handle, HTS221_I2C_ADDRESS, reg,
		I2C_MEMADD_SIZE_8BIT, bufp, len, 1000);
	}
	return 0;
}

/*
 * @brief  Write generic device register (platform dependent)
 *
 * @param  handle    customizable argument. In this examples is used in
 *                   order to select the correct sensor bus handler.
 * @param  reg       register to write
 * @param  bufp      pointer to data to write in register reg
 * @param  len       number of consecutive register to write
 *
 */
static int32_t lps22hb_platform_write(void *handle, uint8_t reg, uint8_t *bufp,
		uint16_t len) {
	if (handle == &hi2c1) {
		/* Write multiple command */
		HAL_I2C_Mem_Write(handle, LPS22HB_I2C_ADD_H, reg,
		I2C_MEMADD_SIZE_8BIT, bufp, len, 1000);
	}
	return 0;
}

/*
 * @brief  Read generic device register (platform dependent)
 *
 * @param  handle    customizable argument. In this examples is used in
 *                   order to select the correct sensor bus handler.
 * @param  reg       register to read
 * @param  bufp      pointer to buffer that store the data read
 * @param  len       number of consecutive register to read
 *
 */
static int32_t lps22hb_platform_read(void *handle, uint8_t reg, uint8_t *bufp,
		uint16_t len) {
	if (handle == &hi2c1) {
		/* Read multiple command */
		HAL_I2C_Mem_Read(handle, LPS22HB_I2C_ADD_H, reg,
		I2C_MEMADD_SIZE_8BIT, bufp, len, 1000);
	}
	return 0;
}

static int32_t lsm303agr_platform_write(void *handle, uint8_t Reg, uint8_t *Bufp,
                              uint16_t len)
{
  uint32_t i2c_add = (uint32_t)handle;
  if (i2c_add == LSM303AGR_I2C_ADD_XL)
  {
    /* enable auto incremented in multiple read/write commands */
    Reg |= 0x80;
  }
  HAL_I2C_Mem_Write(&hi2c1, i2c_add, Reg,
                    I2C_MEMADD_SIZE_8BIT, Bufp, len, 1000);
  return 0;
}

static int32_t lsm303agr_platform_read(void *handle, uint8_t Reg, uint8_t *Bufp,
                             uint16_t len)
{
  uint32_t i2c_add = (uint32_t)handle;
  if (i2c_add == LSM303AGR_I2C_ADD_XL)
  {
    /* enable auto incremented in multiple read/write commands */
    Reg |= 0x80;
  }
  HAL_I2C_Mem_Read(&hi2c1, (uint8_t) i2c_add, Reg,
                   I2C_MEMADD_SIZE_8BIT, Bufp, len, 1000);
  return 0;
}

static int32_t lsm6dsl_platform_write(void *handle, uint8_t Reg, uint8_t *Bufp,
                              uint16_t len)
{
  if (handle == &hi2c1)
  {
    HAL_I2C_Mem_Write(handle, LSM6DSL_I2C_ADD_H, Reg,
                      I2C_MEMADD_SIZE_8BIT, Bufp, len, 1000);
  }
  return 0;
}

static int32_t lsm6dsl_platform_read(void *handle, uint8_t Reg, uint8_t *Bufp,
                             uint16_t len)
{
  if (handle == &hi2c1)
  {
      HAL_I2C_Mem_Read(handle, LSM6DSL_I2C_ADD_H, Reg,
                       I2C_MEMADD_SIZE_8BIT, Bufp, len, 1000);
  }
  return 0;
}
/*
 * @brief  Send buffer to console (platform dependent)
 *
 * @param  tx_buffer     buffer to trasmit
 * @param  len           number of byte to send
 *
 */
static void tx_com(uint8_t *tx_buffer, uint16_t len ) {
  HAL_UART_Transmit(&huart3, tx_buffer, len, 1000);
}

/*
 * @brief  platform specific delay (platform dependent)
 *
 * @param  ms        delay in ms
 *
 */
static void platform_delay(uint32_t ms) {
	HAL_Delay(ms);
}

/*
 * @brief  platform specific initialization (platform dependent)
 */
static void platform_init(void) {
}

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
	/* User can add his own implementation to report the HAL error return state */

  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{ 
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     tex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
